## QB Bootcamp 7 - Notes on unix scripting

## Lets explore your file system a little
$ cd /
$ ls
$ cd Users
$ ls
$ cd <your name>
$ ls
$ cd Desktop
$ touch secret_file.txt

## What do you see on your desktop now?

## how big are the files on your desktop?
$ ls -l

## or in a more human readable way?
$ ls -lh


## Create a work directory
$ mkdir human_analysis
$ ls
$ cd human_analysis/
$ ls

## Create a temp directory
$ mkdir tmp
$ cd tmp

## 'touch' an empty file again
$ touch empty.txt
$ ls -l

## Go back a directory
$ cd ..
$ ls -lR


## Go back to your last directory 
$ cd -

## lets delete the empty file
$ rm empty.txt

## Go back and clean up (be very careful!)
$ cd ..
$ rm -rf tmp 




#############################################################################

## what if we wanted to save something to a file
$ echo "Mike" > mike.txt 
$ echo "Mickey" > mickey.txt
$ echo "Justin" > justin.txt

$ ls -l

## Lets peak at those files
$ cat mike.txt
$ cat mickey.txt
$ cat justin.txt

## Lets merge all the files together
$ cat mike.txt mickey.txt justin.txt > names.txt

## Lets peak at the file
$ cat names.txt
$ head -1 names.txt
$ head -2 names.txt
$ last -1 names.txt

## Notice we gave the command 'head' or 'tail' the option -1 or -2

## Can also use wild cards (head is nice because it shows the file names)
$ cat *.txt
$ head *.txt
$ head m*.txt


## Lets sort the names alphabetically
$ sort names.txt > sorted.txt
$ cat sorted.txt


## Lets count the lines in the files
$ wc -l names.txt


## What if we want to find a word in a file ... 'grep' for it
$ grep M *.txt


## Or lines that *dont* have a keyword 'grep -v'
$ grep -v M *.txt


## A very powerful concept in Unix is the ability to chain commands
## together using 'pipes' (the vertical bar above enter '|')
$ grep -v M *.txt | wc -l


## Now clean up everything
$ rm *.txt


## check out what is running on your computer
$ top

######################################################################


## Lets do some useful science

## Download the annotation of the human genome (curl dash capital O)
$ curl -O http://schatzlab.cshl.edu/teaching/2014/hg19.gff.gz


## See how big it is
$ ls –l
-rw-r--r--  1 mschatz  staff  24904770 Sep  2 22:48 hg19.gff.gz


## See how big it is in a human readable way
$ ls –lh
-rw-r--r--  1 mschatz  staff    24M Sep  2 22:48 hg19.gff.gz


## Make a copy
$ cp hg19.gff.gz hg19.2.gff.gz
$ ls


## Rename the copy with the move command (mv)
## Also used for moving files from one directory to another
$ mv hg19.2.gff.gz hg19_2.gff.gz
$ ls


## delete the copy
$ rm hg13_2.gff.gz
$ ls


# uncompress compressed files with gunzip
$ gunzip hg19.gff.gz
$ ls -lh
total 1065808
-rw-r--r--  1 mschatz  staff   520M Sep  2 22:48 hg19.gff
# Notice it is >20 times large

## look at the first few lines using the command head
$ head hg19.gff
##gff-version 3
#!gff-spec-version 1.20
#!processor NCBI annotwriter
#!genome-build Genome Reference Consortium GRCh37.p13
#!genome-build-accession NCBI_Assembly:GCF_000001405.25
#!annotation-source NCBI Homo sapiens Annotation Release 105
##sequence-region NC_000001.10 1 249250621
##species http://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=9606
NC_000001.10	RefSeq	region	1	249250621	.	+	.	ID=id0;Name=1;Dbxref=taxon:
31267

## How many lines are in the file?
$ wc -l hg19.gff
 1937161 hg19.gff

## page through the file
$ less hg19.gff

## less -S is very useful for "wide" files
$ less -S hg19.gff


## Find just the BestRefSeq annotations
$ grep BestRefSeq hg19.gff | less -S
$ grep BestRefSeq hg19.gff | wc -l
  922144
  

## Save it to a new file
$ grep BestRefSeq hg19.gff > hg19.BestRefSeq.gff

$ ls -lh
total 1599120
-rw-r--r--  1 mschatz  staff   260M Sep  2 23:11 hg19.BestRefSeq.gff
-rw-r--r--  1 mschatz  staff   520M Sep  2 22:48 hg19.gff

## Count the number of genes
$ grep gene hg19.BestRefSeq.gff | wc -l
  922144

## That doesnt look right, lets focus on column 3
$ cut -f3 hg19.BestRefSeq.gff | sort | uniq -c
387524 CDS
458708 exon
26705 gene
38536 mRNA
2729 ncRNA
1620 primary_transcript
  21 rRNA
6301 transcript


## Have to ensure the whole field is gene with tabs on either side
$ grep '\tgene\t' hg19.BestRefSeq.gff | wc -l
   26705

## Save the genes to a file
$ grep '\tgene\t' hg19.BestRefSeq.gff > hg19.BestRefSeq.gene.gff

## try to count genes per chromosomes
$ cut -f1 hg19.BestRefSeq.gff | sort | uniq -c
83839 NC_000001.10
63903 NC_000002.11
49456 NC_000003.11
31388 NC_000004.11
35525 NC_000005.9
41083 NC_000006.11
39584 NC_000007.13
27699 NC_000008.10
34100 NC_000009.11
36886 NC_000010.10
47812 NC_000011.9
48267 NC_000012.11
14236 NC_000013.10
25241 NC_000014.8
27682 NC_000015.9
33043 NC_000016.9
49417 NC_000017.10
13275 NC_000018.9
46230 NC_000019.9
21558 NC_000020.10
9964 NC_000021.8
17884 NC_000022.10
33200 NC_000023.10
4714 NC_000024.9
   8 NT_113885.1
  24 NT_113888.1
  15 NT_113889.1
6383 NT_113891.2
   8 NT_113901.1
  10 NT_113923.1
   3 NT_113930.1
 110 NT_113949.1
   9 NT_113961.1
....

What are all those other things?   


## How many "chromosomes" total
$ cut -f1 hg19.BestRefSeq.gene.gff | sort | uniq -c | sort -nrk1 | wc -l
     217


## Count genes per chromosome
$ cut -f1 hg19.BestRefSeq.gene.gff | sort | uniq -c | sort -nrk1 | head -24
2426 NC_000001.10
1632 NC_000019.9
1510 NC_000002.11
...







A bash script is just a list of commands
$ cat simple_script.sh 
#!/bin/sh

echo "Hello, World”
echo "Shall we play a game?”

## You first have to make the script executable
$ chmod +x simple_script.sh

## And then you have to tell it to try to run out of the current directory
$ ./simple_script.sh



Things get interesting when we add variables and loops
$ cat loop_script.sh 
#!/bin/sh

for chrom in NC_000001.10 NC_000002.11 NC_000003.11
do
  echo Searching $chrom
  grep $chrom hg19.BestRefSeq.gene.gff > $chrom.BestRefSeq.gene.gff
done

$ chmod +x loop_script.sh
$ ./loop_script.pl



## Lets review
$ history


## Challenges
1. Where is TP53 located? Where is NRAS? Where is SRY?

2. How many genes are annotated with “tumor” or “oncogene”?

3. Create a file with the RefSeqGenes for each chromosome and sort them by file size




4. What do these do?

cat
cd
chmod
curl
cut
echo
grep
head
less
ls
mkdir
rm 
rm -rf
sort
tail
uniq
wc -l








